@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2009  Marcel Holtmann <marcel@holtmann.org>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -98,8 +98,6 @@ typedef struct {
 #define BT_NEW_STREAM			2
 #define BT_START_STREAM			3
 #define BT_STOP_STREAM			4
-#define BT_SUSPEND_STREAM		5
-#define BT_RESUME_STREAM		6
 #define BT_CONTROL			7
 
 #define BT_CAPABILITIES_TRANSPORT_A2DP	0
