@@ -271,38 +271,6 @@ static struct csd_call *find_call_with_status(int status)
 	return NULL;
 }
 
-static gboolean update_indicators(gpointer user_data)
-{
-	if (net.status < NETWORK_REG_STATUS_NOSERV) {
-		int signal;
-		telephony_update_indicator(maemo_indicators, "service",
-						EV_SERVICE_PRESENT);
-		signal = telephony_get_indicator(maemo_indicators, "signal");
-		telephony_update_indicator(maemo_indicators, "signal", signal);
-	} else
-		telephony_update_indicator(maemo_indicators, "service",
-						EV_SERVICE_NONE);
-
-	switch (net.status) {
-	case NETWORK_REG_STATUS_HOME:
-		telephony_update_indicator(maemo_indicators, "roam",
-						EV_ROAM_INACTIVE);
-		break;
-	case NETWORK_REG_STATUS_ROAM:
-	case NETWORK_REG_STATUS_ROAM_BLINK:
-		telephony_update_indicator(maemo_indicators, "roam",
-						EV_ROAM_ACTIVE);
-		break;
-	}
-
-	if ((battchg_last > 0 || battchg_design > 0) && battchg_cur >= 0) {
-		int bat = telephony_get_indicator(maemo_indicators, "battchg");
-		telephony_update_indicator(maemo_indicators, "battchg", bat);
-	}
-
-	return FALSE;
-}
-
 static int release_call(struct csd_call *call)
 {
 	DBusMessage *msg;
@@ -444,9 +412,19 @@ static int call_transfer(void)
 
 void telephony_device_connected(void *telephony_device)
 {
+	struct csd_call *coming;
+
 	debug("telephony-maemo: device %p connected", telephony_device);
 
-	g_timeout_add_seconds(1, update_indicators, NULL);
+	coming = find_call_with_status(CSD_CALL_STATUS_MT_ALERTING);
+	if (coming) {
+		if (find_call_with_status(CSD_CALL_STATUS_ACTIVE))
+			telephony_call_waiting_ind(coming->number,
+							NUMBER_TYPE_TELEPHONY);
+		else
+			telephony_incoming_call_ind(coming->number,
+							NUMBER_TYPE_TELEPHONY);
+	}
 }
 
 void telephony_device_disconnected(void *telephony_device)
@@ -661,6 +639,8 @@ static int csd_status_to_hfp(struct csd_call *call)
 		return -1;
 	case CSD_CALL_STATUS_CREATE:
 		return CALL_STATUS_DIALING;
+	case CSD_CALL_STATUS_WAITING:
+		return CALL_STATUS_WAITING;
 	case CSD_CALL_STATUS_PROCEEDING:
 		/* PROCEEDING can happen in outgoing/incoming */
 		if (call->originating)
@@ -796,8 +776,8 @@ void telephony_call_hold_req(void *telephony_device, const char *cmd)
 		}
 		break;
 	case '3':
-		call = find_call_with_status(CSD_CALL_STATUS_HOLD);
-		if (call)
+		if (find_call_with_status(CSD_CALL_STATUS_HOLD) ||
+				find_call_with_status(CSD_CALL_STATUS_WAITING))
 			err = create_conference();
 		break;
 	case '4':
@@ -850,7 +830,12 @@ static void handle_incoming_call(DBusMessage *msg)
 	telephony_update_indicator(maemo_indicators, "callsetup",
 					EV_CALLSETUP_INCOMING);
 
-	telephony_incoming_call_ind(number, NUMBER_TYPE_TELEPHONY);
+	if (find_call_with_status(CSD_CALL_STATUS_ACTIVE))
+		telephony_call_waiting_ind(call->number,
+						NUMBER_TYPE_TELEPHONY);
+	else
+		telephony_incoming_call_ind(call->number,
+						NUMBER_TYPE_TELEPHONY);
 }
 
 static void handle_outgoing_call(DBusMessage *msg)
@@ -959,6 +944,10 @@ static void handle_call_status(DBusMessage *msg, const char *call_path)
 	case CSD_CALL_STATUS_ACTIVE:
 		if (call->on_hold) {
 			call->on_hold = FALSE;
+			/* Set the status already here since otherwise
+			 * find_call_with_status will match this call even
+			 * though it shouldn't */
+			call->status = (int) status;
 			if (find_call_with_status(CSD_CALL_STATUS_HOLD))
 				telephony_update_indicator(maemo_indicators,
 							"callheld",
@@ -1000,6 +989,16 @@ static void handle_call_status(DBusMessage *msg, const char *call_path)
 	case CSD_CALL_STATUS_RECONNECT_PENDING:
 		break;
 	case CSD_CALL_STATUS_TERMINATED:
+		if (call->on_hold) {
+			/* Set the status already here since otherwise
+			 * find_call_with_status will match this call even
+			 * though it shouldn't */
+			call->status = (int) status;
+			if (!find_call_with_status(CSD_CALL_STATUS_HOLD))
+				telephony_update_indicator(maemo_indicators,
+								"callheld",
+								EV_CALLHELD_NONE);
+		}
 		break;
 	case CSD_CALL_STATUS_SWAP_INITIATED:
 		break;
@@ -1030,11 +1029,11 @@ static void get_operator_name_reply(DBusPendingCall *pending_call,
 	}
 
 	dbus_error_init(&err);
-	if (!dbus_message_get_args(reply, NULL,
+	if (!dbus_message_get_args(reply, &err,
 					DBUS_TYPE_STRING, &name,
 					DBUS_TYPE_INT32, &net_err,
 					DBUS_TYPE_INVALID)) {
-		error("Unexpected paramters in get_operator_name reply:",
+		error("Unexpected get_operator_name reply parameters: %s, %s",
 			err.name, err.message);
 		dbus_error_free(&err);
 		goto done;
@@ -1481,7 +1480,8 @@ static void registration_status_reply(DBusPendingCall *call, void *user_data)
 				AG_FEATURE_REJECT_A_CALL |
 				AG_FEATURE_ENHANCED_CALL_STATUS |
 				AG_FEATURE_ENHANCED_CALL_CONTROL |
-				AG_FEATURE_EXTENDED_ERROR_RESULT_CODES;
+				AG_FEATURE_EXTENDED_ERROR_RESULT_CODES |
+				AG_FEATURE_THREE_WAY_CALLING;
 
 	reply = dbus_pending_call_steal_reply(call);
 
