@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2009  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -21,8 +21,6 @@
  *
  */
 
-#define MAX_PATH_LENGTH 64 /* D-Bus path */
-
 #define PANU_UUID	"00001115-0000-1000-8000-00805f9b34fb"
 #define NAP_UUID	"00001116-0000-1000-8000-00805f9b34fb"
 #define GN_UUID		"00001117-0000-1000-8000-00805f9b34fb"
@@ -41,5 +39,3 @@ int bnep_kill_all_connections(void);
 int bnep_connadd(int sk, uint16_t role, char *dev);
 int bnep_if_up(const char *devname, uint16_t id);
 int bnep_if_down(const char *devname);
-
-int read_remote_name(bdaddr_t *src, bdaddr_t *dst, char *buf, size_t size);
