@@ -2,7 +2,7 @@
  *
  *  BlueZ - Bluetooth protocol stack for Linux
  *
- *  Copyright (C) 2004-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2009  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -111,7 +111,7 @@ static struct network_conn *find_connection(GSList *list, uint16_t id)
 static inline DBusMessage *not_supported(DBusMessage *msg)
 {
 	return g_dbus_create_error(msg, ERROR_INTERFACE ".Failed",
-							"Not suported");
+							"Not supported");
 }
 
 static inline DBusMessage *already_connected(DBusMessage *msg)
