@@ -98,10 +98,13 @@ struct btd_adapter {
 	uint32_t discov_timeout;	/* discoverable time(sec) */
 	guint pairable_timeout_id;	/* pairable timeout id */
 	uint32_t pairable_timeout;	/* pairable time(sec) */
-	uint8_t scan_mode;		/* scan mode: SCAN_DISABLED, SCAN_PAGE, SCAN_INQUIRY */
-	uint8_t mode;			/* off, connectable, discoverable, limited */
+	uint8_t scan_mode;		/* scan mode: SCAN_DISABLED, SCAN_PAGE,
+					 * SCAN_INQUIRY */
+	uint8_t mode;			/* off, connectable, discoverable,
+					 * limited */
 	uint8_t global_mode;		/* last valid global mode */
-	int state;			/* standard inq, periodic inq, name resloving */
+	int state;			/* standard inq, periodic inq, name
+					 * resloving */
 	GSList *found_devices;
 	GSList *oor_devices;		/* out of range device list */
 	DBusMessage *discovery_cancel;	/* discovery cancel message request */
@@ -176,20 +179,6 @@ static inline DBusMessage *unsupported_major_class(DBusMessage *msg)
 			"Unsupported Major Class");
 }
 
-static void send_out_of_range(const char *path, GSList *l)
-{
-	while (l) {
-		const char *peer_addr = l->data;
-
-		g_dbus_emit_signal(connection, path,
-				ADAPTER_INTERFACE, "DeviceDisappeared",
-				DBUS_TYPE_STRING, &peer_addr,
-				DBUS_TYPE_INVALID);
-
-		l = l->next;
-	}
-}
-
 static int found_device_cmp(const struct remote_dev_info *d1,
 			const struct remote_dev_info *d2)
 {
@@ -240,7 +229,8 @@ int pending_remote_name_cancel(struct btd_adapter *adapter)
 
 	if (hci_read_remote_name_cancel(dd, &dev->bdaddr,
 					HCI_REQ_TIMEOUT) < 0) {
-		error("Remote name cancel failed: %s(%d)", strerror(errno), errno);
+		error("Remote name cancel failed: %s(%d)", strerror(errno),
+									errno);
 		err = -errno;
 	}
 
@@ -250,10 +240,12 @@ int pending_remote_name_cancel(struct btd_adapter *adapter)
 	adapter->found_devices = NULL;
 
 	hci_close_dev(dd);
+
 	return err;
 }
 
-static int set_limited_discoverable(int dd, const uint8_t *cls, gboolean limited)
+static int set_limited_discoverable(int dd, const uint8_t *cls,
+							gboolean limited)
 {
 	uint32_t dev_class;
 	int num = (limited ? 2 : 1);
@@ -450,18 +442,18 @@ static int set_mode(struct btd_adapter *adapter, uint8_t new_mode)
 			hci_close_dev(dd);
 			return err;
 		}
-	} else {
-		/* discoverable or limited */
-		if ((scan_enable & SCAN_INQUIRY) && (new_mode != adapter->mode)) {
-			adapter_remove_discov_timeout(adapter);
-			if (adapter->discov_timeout)
-				adapter_set_discov_timeout(adapter,
+	} else if ((scan_enable & SCAN_INQUIRY) &&
+						(new_mode != adapter->mode)) {
+		adapter_remove_discov_timeout(adapter);
+		if (adapter->discov_timeout)
+			adapter_set_discov_timeout(adapter,
 						adapter->discov_timeout);
-			if (new_mode == MODE_LIMITED)
-				set_limited_discoverable(dd, adapter->dev.class, TRUE);
-			else if (adapter->mode == MODE_LIMITED)
-				set_limited_discoverable(dd, adapter->dev.class, FALSE);
-		}
+		if (new_mode == MODE_LIMITED)
+			set_limited_discoverable(dd, adapter->dev.class,
+									TRUE);
+		else if (adapter->mode == MODE_LIMITED)
+			set_limited_discoverable(dd, adapter->dev.class,
+									FALSE);
 	}
 done:
 	modestr = mode2str(new_mode);
@@ -573,8 +565,8 @@ static void adapter_set_pairable_timeout(struct btd_adapter *adapter,
 		return;
 
 	adapter->pairable_timeout_id = g_timeout_add_seconds(interval,
-							pairable_timeout_handler,
-							adapter);
+						pairable_timeout_handler,
+						adapter);
 }
 
 static struct session_req *find_session(GSList *list, const char *sender)
@@ -635,6 +627,11 @@ static void session_remove(struct session_req *req)
 
 		debug("Stopping discovery");
 
+		g_slist_foreach(adapter->found_devices, (GFunc) dev_info_free,
+				NULL);
+		g_slist_free(adapter->found_devices);
+		adapter->found_devices = NULL;
+
 		if (adapter->state & STD_INQUIRY)
 			cancel_discovery(adapter);
 		else if (adapter->scheduler_id)
@@ -860,7 +857,7 @@ static DBusMessage *set_name(DBusConnection *conn, DBusMessage *msg,
 	const char *path;
 
 	if (!g_utf8_validate(name, -1, NULL)) {
-		error("Name change failed: the supplied name isn't valid UTF-8");
+		error("Name change failed: supplied name isn't valid UTF-8");
 		return invalid_args(msg);
 	}
 
@@ -876,7 +873,8 @@ static DBusMessage *set_name(DBusConnection *conn, DBusMessage *msg,
 	return dbus_message_new_method_return(msg);
 }
 
-struct btd_device *adapter_find_device(struct btd_adapter *adapter, const char *dest)
+struct btd_device *adapter_find_device(struct btd_adapter *adapter,
+							const char *dest)
 {
 	struct btd_device *device;
 	GSList *l;
@@ -884,8 +882,8 @@ struct btd_device *adapter_find_device(struct btd_adapter *adapter, const char *
 	if (!adapter)
 		return NULL;
 
-	l = g_slist_find_custom(adapter->devices,
-				dest, (GCompareFunc) device_address_cmp);
+	l = g_slist_find_custom(adapter->devices, dest,
+					(GCompareFunc) device_address_cmp);
 	if (!l)
 		return NULL;
 
@@ -895,7 +893,7 @@ struct btd_device *adapter_find_device(struct btd_adapter *adapter, const char *
 }
 
 struct btd_device *adapter_find_connection(struct btd_adapter *adapter,
-					uint16_t handle)
+						uint16_t handle)
 {
 	GSList *l;
 
@@ -929,7 +927,8 @@ static void adapter_update_devices(struct btd_adapter *adapter)
 }
 
 struct btd_device *adapter_create_device(DBusConnection *conn,
-				struct btd_adapter *adapter, const char *address)
+						struct btd_adapter *adapter,
+						const char *address)
 {
 	struct btd_device *device;
 	const char *path;
@@ -988,7 +987,8 @@ void adapter_remove_device(DBusConnection *conn, struct btd_adapter *adapter,
 }
 
 struct btd_device *adapter_get_device(DBusConnection *conn,
-				struct btd_adapter *adapter, const gchar *address)
+						struct btd_adapter *adapter,
+						const gchar *address)
 {
 	struct btd_device *device;
 
@@ -1242,7 +1242,8 @@ static DBusMessage *get_properties(DBusConnection *conn,
 		struct btd_device *dev = l->data;
 		devices[i] = (char *) device_get_path(dev);
 	}
-	dict_append_array(&dict, "Devices", DBUS_TYPE_OBJECT_PATH, &devices, i);
+	dict_append_array(&dict, "Devices", DBUS_TYPE_OBJECT_PATH,
+								&devices, i);
 	g_free(devices);
 
 	dbus_message_iter_close_container(&iter, &dict);
@@ -1434,7 +1435,7 @@ static DBusMessage *cancel_device_creation(DBusConnection *conn,
 						DBusMessage *msg, void *data)
 {
 	struct btd_adapter *adapter = data;
-	const gchar *address;
+	const gchar *address, *sender = dbus_message_get_sender(msg);
 	struct btd_device *device;
 
 	if (dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &address,
@@ -1450,8 +1451,7 @@ static DBusMessage *cancel_device_creation(DBusConnection *conn,
 				ERROR_INTERFACE ".DoesNotExist",
 				"Device does not exist");
 
-	if (!device_is_temporary(device) ||
-			!device_is_bonding(device, dbus_message_get_sender(msg)))
+	if (!device_is_temporary(device) || !device_is_bonding(device, sender))
 		return not_authorized(msg);
 
 	adapter_remove_device(conn, adapter, device);
@@ -1797,7 +1797,8 @@ static int adapter_setup(struct btd_adapter *adapter, int dd)
 			events[6] |= 0x20;	/* Simple Pairing Complete */
 			events[7] |= 0x04;	/* User Passkey Notification */
 			events[7] |= 0x08;	/* Keypress Notification */
-			events[7] |= 0x10;	/* Remote Host Supported Features Notification */
+			events[7] |= 0x10;	/* Remote Host Supported
+						 * Features Notification */
 		}
 
 		hci_send_cmd(dd, OGF_HOST_CTL, OCF_SET_EVENT_MASK,
@@ -1882,10 +1883,12 @@ static void load_devices(struct btd_adapter *adapter)
 	ba2str(&adapter->bdaddr, srcaddr);
 
 	create_name(filename, PATH_MAX, STORAGEDIR, srcaddr, "profiles");
-	textfile_foreach(filename, create_stored_device_from_profiles, adapter);
+	textfile_foreach(filename, create_stored_device_from_profiles,
+								adapter);
 
 	create_name(filename, PATH_MAX, STORAGEDIR, srcaddr, "linkkeys");
-	textfile_foreach(filename, create_stored_device_from_linkkeys, adapter);
+	textfile_foreach(filename, create_stored_device_from_linkkeys,
+								adapter);
 }
 
 static void load_drivers(struct btd_adapter *adapter)
@@ -2102,7 +2105,8 @@ int adapter_start(struct btd_adapter *adapter)
 		return err;
 	}
 
-	if (hci_read_local_name(dd, sizeof(name), name, HCI_REQ_TIMEOUT) < 0) {
+	if (hci_read_local_name(dd, sizeof(name), name,
+						HCI_REQ_TIMEOUT) < 0) {
 		err = -errno;
 		error("Can't read local name on %s: %s (%d)",
 					adapter->path, strerror(err), err);
@@ -2128,8 +2132,8 @@ int adapter_start(struct btd_adapter *adapter)
 	}
 
 setup:
-	hci_send_cmd(dd, OGF_LINK_POLICY,
-				OCF_READ_DEFAULT_LINK_POLICY, 0, NULL);
+	hci_send_cmd(dd, OGF_LINK_POLICY, OCF_READ_DEFAULT_LINK_POLICY,
+								0, NULL);
 
 	if (hci_test_bit(HCI_INQUIRY, &di.flags))
 		adapter->state |= STD_INQUIRY;
@@ -2334,8 +2338,7 @@ struct btd_adapter *adapter_create(DBusConnection *conn, int id,
 
 	adapter = g_try_new0(struct btd_adapter, 1);
 	if (!adapter) {
-		error("Failed to alloc memory to D-Bus path register data (%s)",
-				path);
+		error("adapter_create: failed to alloc memory for %s", path);
 		return NULL;
 	}
 
@@ -2380,7 +2383,6 @@ void adapter_remove(struct btd_adapter *adapter)
 
 done:
 	g_dbus_unregister_interface(connection, path, ADAPTER_INTERFACE);
-
 	g_free(path);
 }
 
@@ -2414,20 +2416,12 @@ void adapter_set_state(struct btd_adapter *adapter, int state)
 		discov_active = TRUE;
 	else if (adapter->disc_sessions && main_opts.inqmode)
 		adapter->scheduler_id = g_timeout_add_seconds(main_opts.inqmode,
-				(GSourceFunc) start_inquiry, adapter);
+						(GSourceFunc) start_inquiry,
+						adapter);
 
-	if (!discov_active && adapter->found_devices) {
-		g_slist_foreach(adapter->found_devices,
-					(GFunc) dev_info_free, NULL);
-		g_slist_free(adapter->found_devices);
-		adapter->found_devices = NULL;
-	}
-
-	if (!discov_active && adapter->oor_devices) {
-		g_slist_foreach(adapter->oor_devices, (GFunc) g_free, NULL);
-		g_slist_free(adapter->oor_devices);
-		adapter->oor_devices = NULL;
-	}
+	/* Send out of range */
+	if (!discov_active)
+		adapter_update_oor_devices(adapter);
 
 	emit_property_changed(connection, path,
 				ADAPTER_INTERFACE, "Discovering",
@@ -2488,10 +2482,10 @@ int adapter_add_found_device(struct btd_adapter *adapter, bdaddr_t *bdaddr,
 			dev->alias = g_strdup(alias);
 		}
 
-		 /* Get remote name can be received while inquiring.
-		  * Keep in mind that multiple inquiry result events can
-		  * be received from the same remote device.
-		  */
+		/* Get remote name can be received while inquiring.  Keep in
+		 * mind that multiple inquiry result events can be received
+		 * from the same remote device.
+		 */
 		if (name_status != NAME_NOT_REQUIRED)
 			dev->name_status = name_status;
 
@@ -2527,30 +2521,46 @@ int adapter_remove_found_device(struct btd_adapter *adapter, bdaddr_t *bdaddr)
 	if (!dev)
 		return -1;
 
-	adapter->found_devices = g_slist_remove(adapter->found_devices, dev);
-	dev_info_free(dev);
+	dev->name_status = NAME_NOT_REQUIRED;
 
 	return 0;
 }
 
 void adapter_update_oor_devices(struct btd_adapter *adapter)
 {
-	GSList *l = adapter->found_devices;
+	GSList *l;
 	struct remote_dev_info *dev;
 	bdaddr_t tmp;
 
-	send_out_of_range(adapter->path, adapter->oor_devices);
+	for (l = adapter->oor_devices; l; l = l->next) {
+		char *address = l->data;
+		struct remote_dev_info match;
+
+		memset(&match, 0, sizeof(struct remote_dev_info));
+		str2ba(address, &match.bdaddr);
+
+		g_dbus_emit_signal(connection, adapter->path,
+				ADAPTER_INTERFACE, "DeviceDisappeared",
+				DBUS_TYPE_STRING, &address,
+				DBUS_TYPE_INVALID);
+
+		dev = adapter_search_found_devices(adapter, &match);
+		if (!dev)
+			continue;
+
+		adapter->found_devices = g_slist_remove(adapter->found_devices, dev);
+		dev_info_free(dev);
+	}
 
 	g_slist_foreach(adapter->oor_devices, (GFunc) free, NULL);
 	g_slist_free(adapter->oor_devices);
 	adapter->oor_devices = NULL;
 
-	while (l) {
+	for (l = adapter->found_devices; l; l = l->next) {
 		dev = l->data;
 		baswap(&tmp, &dev->bdaddr);
 		adapter->oor_devices = g_slist_append(adapter->oor_devices,
 							batostr(&tmp));
-		l = l->next;
 	}
 }
 
@@ -2685,7 +2695,7 @@ void adapter_remove_connection(struct btd_adapter *adapter,
 	hci_req_queue_remove(adapter->dev_id, &bdaddr);
 
 	if (device_is_authenticating(device))
-		device_cancel_authentication(device);
+		device_cancel_authentication(device, TRUE);
 
 	if (device_is_temporary(device)) {
 		const char *path = device_get_path(device);
@@ -2734,7 +2744,8 @@ void btd_unregister_adapter_driver(struct btd_adapter_driver *driver)
 	adapter_drivers = g_slist_remove(adapter_drivers, driver);
 }
 
-static void agent_auth_cb(struct agent *agent, DBusError *derr, void *user_data)
+static void agent_auth_cb(struct agent *agent, DBusError *derr,
+							void *user_data)
 {
 	struct service_auth *auth = user_data;
 
@@ -2743,8 +2754,10 @@ static void agent_auth_cb(struct agent *agent, DBusError *derr, void *user_data)
 	g_free(auth);
 }
 
-static int btd_adapter_authorize(struct btd_adapter *adapter, const bdaddr_t *dst,
-		const char *uuid, service_auth_cb cb, void *user_data)
+static int btd_adapter_authorize(struct btd_adapter *adapter,
+					const bdaddr_t *dst,
+					const char *uuid,
+					service_auth_cb cb, void *user_data)
 {
 	struct service_auth *auth;
 	struct btd_device *device;
